package com.huiju.welcome.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.ISysUserService;
import com.huiju.welcome.utils.Encrypt;
import com.huiju.welcome.utils.StatusUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@RestController
@RequestMapping("/")
public class SysUserController extends BaseController {
    @Autowired
    private ISysUserService sysUserService;

    @ApiOperation(value = "系统登录", notes = "系统登录")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "form", dataType = "String", name = "username", value = "登录用户名"),
            @ApiImplicitParam(paramType = "form", dataType = "String", name = "password", value = "登录密码, 需要 md5 加密")
    })
    @RequestMapping(value = "/login", method = RequestMethod.POST)
    public ResponseBean login(@RequestParam String username, @RequestParam String password, HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        HttpSession httpSession = request.getSession();

        // 重复登录
        String token = (String) httpSession.getAttribute("token");
        if (null != token && token.equals(username)) {
            responseBean.addSuccess("请不要重复登录");
            return responseBean;
        }

        SysUser sysUser = sysUserService.getDetailByLoginname(username);
        if (null == sysUser) {
            responseBean.addError(HttpStatus.SC_BAD_REQUEST, "账户或者密码不正确");
            return responseBean;
        }

        // 人员被删除
        if (sysUser.getStatus().intValue() == StatusUtils.Delete) {
            responseBean.addError(HttpStatus.SC_BAD_REQUEST, "账户或者密码不正确");
            return responseBean;
        }

        // 密码校验
        if (!checkPassword(password, sysUser)) {
            responseBean.addError(HttpStatus.SC_BAD_REQUEST, "账户或者密码不正确");
            return responseBean;
        }

        if (StatusUtils.Normal != sysUser.getStatus().intValue()) {
            responseBean.addError(HttpStatus.SC_FORBIDDEN, "账户状态异常, 请联系管理人员");
            return responseBean;
        }

        // 成功之后写 session
        httpSession.setAttribute("token", username);
        httpSession.setAttribute("token-id", sysUser.getUserId());
        responseBean.addSuccess("登录成功");
        return responseBean;
    }

    @ApiOperation(value = "系统登出", notes = "系统登出")
    @RequestMapping(value = "/logout", method = RequestMethod.POST)
    public ResponseBean logout(HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        HttpSession httpSession = request.getSession();

        httpSession.removeAttribute("token");
        httpSession.removeAttribute("token-id");
        responseBean.addSuccess("登出成功");
        return responseBean;
    }

    @ApiOperation(value = "获取用户信息", notes = "获取用户信息")
    @RequestMapping(value = "/user", method = RequestMethod.GET)
    public ResponseBean user(HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        HttpSession httpSession = request.getSession();

        Integer userId = (Integer) httpSession.getAttribute("token-id");

        Map<String, Object> result = sysUserService.getProfileWithRights(userId);

        if (null == result) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "用户信息异常");
        } else {
            responseBean.addSuccess(result);
        }

        return responseBean;
    }

    @ApiOperation(value = "获取用户信息", notes = "获取用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageNum", value = "当前页"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageSize", value = "每页数量"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "name", value = "姓名"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "phone", value = "手机号"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "identity", value = "用户身份"),
    })
    @RequestMapping(value = "/sysuser", method = RequestMethod.GET)
    public ResponseBean list(
            @RequestParam Integer pageNum,
            @RequestParam  Integer pageSize,
            String name,
            String phone,
            String identity,
            HttpServletRequest request
    ) {
        ResponseBean responseBean = new ResponseBean();

        Page<SysUser> pg = new Page<>();
        pg.setCurrent(pageNum);
        pg.setSize(pageSize);
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("create_date");
        queryWrapper.gt("user_id", 1); // 去掉管理员

        if (null != name && !"".equals(name)) {
            queryWrapper.like("username", name);
        }

        if (null != phone && !"".equals(phone)) {
            queryWrapper.eq("phone", phone);
        }

        if (null != identity ) {
            queryWrapper.eq("user_identity", identity);
        }

        // 取消传递密码字段
        IPage<SysUser> nwPg = sysUserService.page(pg, queryWrapper);
        List<SysUser> usrList = nwPg.getRecords();
        for (SysUser usr : usrList) {
            usr.setPassword("");
        }

        responseBean.addSuccess(nwPg);
        return responseBean;
    }

    @ApiOperation(value = "获取用户信息", notes = "获取用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataType = "int", name = "id", value = "用户详情")
    })
    @RequestMapping(value = "/sysuser/{id}", method = RequestMethod.GET)
    public ResponseBean get(@PathVariable Integer id, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        if (id.intValue() < 2) {
            responseBean.addError(HttpStatus.SC_NOT_FOUND, "错误的用户ID");
            return responseBean;
        }

        SysUser sysUser = sysUserService.getById(id);
        sysUser.setPassword("");

        responseBean.addSuccess(sysUser);
        return responseBean;
    }

    @ApiOperation(value = "新增人员", notes = "新增人员")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "SysUser", name = "sysUser", value = "用户详情")
    })
    @RequestMapping(value = "/sysuser", method = RequestMethod.POST)
    public ResponseBean create(@RequestBody SysUser sysUser, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        sysUser.setCreateDate(LocalDateTime.now());

        boolean success = sysUserService.save(sysUser);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "新增用户失败");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "编辑人员", notes = "编辑人员")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "SysUser", name = "sysUser", value = "用户详情")
    })
    @RequestMapping(value = "/sysuser", method = RequestMethod.PUT)
    public ResponseBean update(@RequestBody SysUser sysUser, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        boolean success = sysUserService.updateById(sysUser);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "更新用户失败");
            return responseBean;
        }

        return responseBean;
    }

    @GetMapping("/miniapp/user/of/{identity}")
    public ResponseBean specUserList(@PathVariable("identity") String userIdentity) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_identity", userIdentity);
        queryWrapper.eq("status", StatusUtils.Normal);

        List<SysUser> userList = sysUserService.list(queryWrapper);
        if (null != userList && userList.size() > 0) {
            for (SysUser user: userList) {
                user.setPassword(null);
            }
        }

        return ResponseBean.success(userList);
    }

    private boolean checkPassword(String password, SysUser user)  {
        String passwd = Encrypt.md5(password, user.getUserId().toString());
        if (null == passwd || !passwd.equals(user.getPassword())) {
            return false;
        }

        return true;
    }

    @ApiOperation(value = "员工管理列表", notes = "员工管理列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "userName", value = "用户姓名"),
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "phone", value = "电话")
    })
    @RequestMapping(value = "/userManagement", method = RequestMethod.GET)
    public ResponseBean userManagement(@RequestParam(defaultValue = "1") int pageNum,
                                       @RequestParam(defaultValue = "10") int pageSize,
                                       @RequestParam(value = "userName",required = false) String userName,
                                       @RequestParam(value = "phone",required = false) String phone) {
        IPage<SysUserRole> pg = new Page<>(pageNum, pageSize);

        ResponseBean userManagement= sysUserService.userManagementList(pg,userName,phone);

        return userManagement;
    }

    @ApiOperation(value = "员工管理角色权限添加", notes = "员工管理角色权限添加")
    @RequestMapping(value = "/addUserRole", method = RequestMethod.POST)
    public ResponseBean userManagement(@RequestParam(value = "userId",required = false) String userId,
                                       @RequestParam(value = "roleId",required = false) List<String> roleId) {
        ResponseBean userManagement= sysUserService.addUserRole(userId,roleId);
        return userManagement;
    }

    @ApiOperation(value = "查询当前员工的角色权限", notes = "查询当前员工的角色权限")
    @RequestMapping(value = "/userRoledetails", method = RequestMethod.GET)
    public ResponseBean userRoledetails(@RequestParam(value = "userId",required = false) Integer userId) {
         ResponseBean userRoledetails= sysUserService.userRoledetails(userId);
        return userRoledetails;
    }
    @ApiOperation(value = "修改当前员工的角色权限", notes = "修改当前员工的角色权限")
    @RequestMapping(value = "/updateUserRolede", method = RequestMethod.POST)
    public ResponseBean updateUserRolede(@RequestParam(value = "userId",required = false) Integer userId,
                                         @RequestParam(value = "menuArray",required = false) List<String> menuArray) {
        ResponseBean updateUserRolede= sysUserService.updateUserRolede(userId,menuArray);
        return updateUserRolede;
    }
    @ApiOperation(value = "获取所有角色权限", notes = "获取所有角色权限")
    @RequestMapping(value = "/roleAll", method = RequestMethod.GET)
    public ResponseBean roleAll() {
        ResponseBean roleAll= sysUserService.roleAll();
        return roleAll;
    }


    @ApiOperation(value = "角色管理列表", notes = "角色管理列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "userName", value = "角色名称"),
})
    @RequestMapping(value = "/user/roleManagementList", method = RequestMethod.GET)
    public ResponseBean roleManagementList(@RequestParam(defaultValue = "1") int pageNum,
                                       @RequestParam(defaultValue = "10") int pageSize,
                                       @RequestParam(value = "roleName",required = false) String roleName) {
        IPage<SysRole> pg = new Page<>(pageNum, pageSize);
        ResponseBean roleManagementList= sysUserService.roleManagementList(pg,roleName);
        return roleManagementList;
    }

    @ApiOperation(value = "删除当前员工的角色权限", notes = "删除当前员工的角色权限")
    @RequestMapping(value = "/deleteUserRole", method = RequestMethod.POST)
    public ResponseBean deleteUserRole(@RequestParam(value = "userId",required = false) Integer userId) {
        ResponseBean deleteUserRole= sysUserService.deleteUserRole(userId);
        return deleteUserRole;
    }

    @ApiOperation(value = "菜单列表", notes = "菜单列表")
    @RequestMapping(value = "/sysmenu", method = RequestMethod.GET)
    public ResponseBean sysmenu(@RequestParam(defaultValue = "1") int pageNum,
                                       @RequestParam(defaultValue = "10") int pageSize) {
        IPage<SysMenu> pg = new Page<>(pageNum, pageSize);
        ResponseBean sysmenu= sysUserService.sysmenu(pg);
        return sysmenu;
    }

    @ApiOperation(value = "操作菜单", notes = "操作菜单")
    @RequestMapping(value = "/sysmenuAdd", method = RequestMethod.POST)
    public ResponseBean sysmenuAdd(@RequestParam(value = "menuId",required = false) String menuId,
                                   @RequestParam(value = "menuName",required = false) String menuName,
                                   @RequestParam(value = "Integer",required = false) String code) {
        ResponseBean sysmenu= sysUserService.sysmenuAdd(menuId,menuName,code);
        return sysmenu;
    }
    @ApiOperation(value = "當前員工離職", notes = "當前員工離職")
    @RequestMapping(value = "/fireUser", method = RequestMethod.POST)
    public ResponseBean fireUser(@RequestParam(value = "userId",required = false) Integer userId) {
        ResponseBean fireUser= sysUserService.fireUser(userId);
        return fireUser;
    }
}
