package com.huiju.welcome.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.DateUtils;
import com.huiju.welcome.utils.JWTUtils;
import com.huiju.welcome.utils.StatusUtils;
import org.apache.http.HttpStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-06-12
 */
@RestController
@RequestMapping("/")
public class TaCustomerController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaCustomerController.class);

    @Autowired
    public ITaCustomerService iTaCustomerService;

    @Autowired
    ITaWechatUserService iTaWechatUserService;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    ITaMainUsherRecordService iTaMainUsherRecordService;

    @Autowired
    ITaCustomerMapService iTaCustomerMapService;

    @Autowired
    ISysUserService iSysUserService;

    /**
     * 保存对象
     * @param taCustomer 实体对象
     * @return
     */
    @RequestMapping(value="/miniapp/customer",method= RequestMethod.POST)
    public ResponseBean save(@RequestBody String paramStr, HttpServletRequest request){
        JSONObject params = JSONObject.parseObject(paramStr);
        if (null == params) {
            return ResponseBean.error("保存内容格式非法或者为空", HttpStatus.SC_BAD_REQUEST);
        }

        TaCustomer taCustomer = JSONObject.parseObject(paramStr, TaCustomer.class);
        JSONArray follows = params.getJSONArray("follows");

        String userId = JWTUtils.getSubject(request);
        SysUser user = iSysUserService.getById(iTaWechatUserService.getById(userId).getPersonId());
        taCustomer.setConsultantId(user.getUserId());
        taCustomer.setRealtyConsultant(user.getUsername());

        ResponseBean responseBean = new ResponseBean();
        try {
            // 会员保存之前, 先保存人员
            TaPerson taPerson = iTaPersonService.newPersonByFront(taCustomer);
            if (null == taPerson) {
                responseBean.addError("预添加人员信息失败");
                return responseBean;
            }

            // 客户ID 与 人员ID 一致
            taCustomer.setCustomerId(taPerson.getPersonId());

            // 保存会员与人员映射
            List<TaCustomerMap> customerMapList = new ArrayList<>();
            TaCustomerMap custMap = new TaCustomerMap();
            custMap.setCustomerId(taCustomer.getCustomerId());
            custMap.setPersonId(taPerson.getPersonId());
            customerMapList.add(custMap);

            if (null != follows && follows.size() > 0) {
                for (int i = 0; i < follows.size(); i ++) {
                    JSONObject it = follows.getJSONObject(i);
                    Integer perId = it.getInteger("personId");
                    if (perId.equals(taCustomer.getCustomerId())) {
                        continue;
                    }

                    TaCustomerMap foo = new TaCustomerMap();
                    foo.setCustomerId(taCustomer.getCustomerId());
                    foo.setPersonId(perId);
                    customerMapList.add(foo);
                }
            }
            iTaCustomerMapService.saveBatch(customerMapList);

            // 新增会员来访信息
            iTaMainUsherRecordService.newRecordByConsultant(taCustomer, follows);

            taCustomer.setStatus(StatusUtils.Normal);
            taCustomer.setCreateDate(LocalDateTime.now());
            taCustomer.setRemark(null);
            if (iTaCustomerService.save(taCustomer)){
                List<TaPerson> group = iTaCustomerService.getGroup(taCustomer.getCustomerId());
                taCustomer.setGroup(group);
                responseBean.addSuccess(taCustomer);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taCustomerAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/customer/{id}", method= RequestMethod.DELETE)
    public ResponseBean delete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaCustomerService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taCustomerDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param taCustomer 实体对象
     * @return
     */
    @RequestMapping(value="/miniapp/customer/{id}",method= RequestMethod.PUT)
    public ResponseBean update(@RequestBody String paramStr, @PathVariable Integer id, HttpServletRequest request){
        JSONObject params = JSONObject.parseObject(paramStr);
        if (null == params) {
            return ResponseBean.error("保存内容格式非法或者为空", HttpStatus.SC_BAD_REQUEST);
        }

        TaCustomer taCustomer = JSONObject.parseObject(paramStr, TaCustomer.class);
        JSONArray follows = params.getJSONArray("follows");

        ResponseBean responseBean = new ResponseBean();
        try {

            // 保存会员与人员映射
            if (null != follows && follows.size() > 0) {
                List<TaCustomerMap> customerMapList = new ArrayList<>();
                for (int i = 0; i < follows.size(); i ++) {
                    JSONObject it = follows.getJSONObject(i);
                    Integer perId = it.getInteger("personId");
                    if (perId.equals(taCustomer.getCustomerId())) {
                        continue;
                    }

                    TaCustomerMap foo = new TaCustomerMap();
                    foo.setCustomerId(taCustomer.getCustomerId());
                    foo.setPersonId(perId);
                    customerMapList.add(foo);
                }
                iTaCustomerMapService.saveBatch(customerMapList);
            }

//            String remark = taCustomer.getRemark();
//            TaCustomer origin = iTaCustomerService.getById(id);
//            String originRemark = origin.getRemark();
//            if (null != remark && !"".equals(remark.trim())) {
//                originRemark = (null == originRemark || "".equals(originRemark)) ? "" : originRemark + "\n\n";
//                remark = originRemark + "---" + DateUtils.cutSecond(LocalDateTime.now())+ "---" + "\n" + remark;
//                taCustomer.setRemark(remark);
//            } else {
//                taCustomer.setRemark(originRemark);
//            }

            iTaMainUsherRecordService.updateByCustomer(taCustomer, follows, JWTUtils.getSubject(request));

            taCustomer.setRemark(null);
            if (iTaCustomerService.updateById(taCustomer)){

                // 反更新人员
                iTaPersonService.updateByCustomer(taCustomer);

                List<TaPerson> group = iTaCustomerService.getGroup(taCustomer.getCustomerId());
                taCustomer.setGroup(group);

                responseBean.addSuccess(taCustomer);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taCustomerUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/miniapp/customer/{id}",method= RequestMethod.GET)
    public ResponseBean get(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            TaCustomer cust = iTaCustomerService.getWithLastVisit(id);
            if (cust == null) {
                responseBean.addError("未找到客户信息");
                return responseBean;
            }

            List<TaPerson> group = iTaCustomerService.getGroup(cust.getCustomerId());
            cust.setGroup(group);

            responseBean.addSuccess(cust);
        }catch (Exception e){
            logger.error("taCustomerGetByID -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    @GetMapping("/miniapp/history/customer")
    public ResponseBean history(String phone) {
        Map<String, Object> filters = new HashMap<String, Object>(){{
            put("phone", phone);
        }};

        return ResponseBean.success(iTaCustomerService.getHistoryRecBy(filters));
    }


    /**
     * 分页查询
     */
    @RequestMapping(value="/miniapp/customer",method= RequestMethod.GET)
    public ResponseBean list(
        @RequestParam(defaultValue = "1") int pageNumber,
        @RequestParam(defaultValue = "10") int pageSize,
        @RequestParam(defaultValue = "false") boolean mine,
        @RequestParam(defaultValue = "") String nameOrPhone,
        HttpServletRequest request
    ){
        ResponseBean responseBean = new ResponseBean();
        try {
            Integer consultantId = null;
            if (mine) {
                String userId = JWTUtils.getSubject(request);
                TaWechatUser taWechatUser = iTaWechatUserService.getById(Integer.valueOf(userId));
                consultantId = taWechatUser.getPersonId();
            }

            IPage<TaCustomer> result = iTaCustomerService.getCustomerListOfFront(pageNumber, pageSize, consultantId, nameOrPhone);
            List<TaCustomer> custs = result.getRecords();

            if (null != custs && custs.size() > 0) {
                for (TaCustomer cust: custs) {
                    cust.setLastVisit(iTaCustomerService.getLastVisitInfo(cust.getCustomerId()));
                }
            }

            responseBean.addSuccess(result);
        }catch (Exception e){
            logger.error("taCustomerList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
