package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.mapper.*;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.ISysUserService;
import com.huiju.welcome.utils.StatusUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements ISysUserService {

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysMenuMapper sysMenuMapper;

    @Autowired
    private SysRoleMenuMapper sysRoleMenuMapper;

    @Autowired
    private SysUserRoleMapper sysUserRoleMapper;

    @Autowired
    private SysRoleMapper sysRoleMapper;

    @Override
    public SysUser getDetailByLoginname(String username) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("loginname",username);
        return sysUserMapper.selectOne(queryWrapper);
    }

    @Override
    public SysUser getByPhone(String phone) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone",phone);
        queryWrapper.eq("status", StatusUtils.Normal);
        queryWrapper.last("limit 1");
        return sysUserMapper.selectOne(queryWrapper);
    }

    @Override
    public ResponseBean userRoleTree() {
        return null;
    }

    @Override
    public ResponseBean userManagementList(IPage<SysUserRole> pg,String userName, String phone) {
        ResponseBean responseBean= new ResponseBean();
        IPage<SysUserRole> list= sysUserRoleMapper.userManagementList(pg,userName,phone);
        responseBean.addSuccess(list);
        return responseBean;
    }

    @Override
    public ResponseBean roleManagementList(IPage<SysRole> pg, String roleName) {
        ResponseBean responseBean= new ResponseBean();
        IPage<SysRole> list= sysRoleMapper.roleManagementList(pg,roleName);
        responseBean.addSuccess(list);
        return responseBean;
    }

    @Override
    public Map<String, Object> getProfileWithRights(Integer userId) {
        SysUser user = sysUserMapper.getUserWithAuthority(userId);
        if (null == user) return null;
        user.setPassword(null);

        List<SysMenu> menus = sysMenuMapper.getAllMenusWithAuthority();

        return new HashMap<String, Object>(){{
            put("user", user);
            put("menus", menus);
        }};
    }
    @Override
    public ResponseBean addUserRole(String userId, List<String> role) {
        ResponseBean responseBean = new ResponseBean<>();
        String[] strArray = role.toArray(new String[role.size()]);
        QueryWrapper<SysUser> sysUserQueryWrapper= new QueryWrapper<>();
        sysUserQueryWrapper.eq("user_id",userId);
        SysUser sysUser = sysUserMapper.selectOne(sysUserQueryWrapper);
        // 查询当前人员角色权限是否存在
        List<SysUserRole> sysUserRole= sysUserRoleMapper.selectListSysUserRole(sysUser.getUserId());
        if (sysUserRole!=null && !sysUserRole.isEmpty()){
            responseBean.addError("此用户已存在权限，请点击修改");
            return responseBean;
        }
        for (String id:strArray){
            SysUserRole userRole= new SysUserRole();
            userRole.setUserId(sysUser.getUserId()).setRoleId(Integer.valueOf(id));
            sysUserRoleMapper.insert(userRole);
        }
        responseBean.addSuccess(sysUser);
        return responseBean;
    }

    @Override
    public ResponseBean userRoledetails(Integer userId) {
        ResponseBean responseBean= new ResponseBean<>();
        List<Integer> arrayList= new ArrayList<>();
        SysUser user= sysUserMapper.selectById(userId);
        List<SysRole> sysRole= sysRoleMapper.selectroleAll();
        List<SysUserRole> sysUserRoleList= sysUserRoleMapper.selectListSysUserRole(userId);
        for (SysUserRole sysUserRole:sysUserRoleList){
            arrayList.add(sysUserRole.getRoleId());
        }
        HashMap<Object,Object> map = new HashMap<>();
        map.put("arrayList",arrayList);
        map.put("user",user);
        map.put("sysRole",sysRole);
        responseBean.addSuccess(map);
        return responseBean;
    }

    @Override
    public ResponseBean updateUserRolede(Integer userId,List<String> role) {
        ResponseBean responseBean= new ResponseBean<>();
        // 查出当前用户的所有权限，先删除后添加
        List<SysUserRole> sysUserRoleList= sysUserRoleMapper.selectListSysUserRole(userId);
        for (SysUserRole sysUserRole:sysUserRoleList){
            sysUserRoleMapper.delete(new QueryWrapper<SysUserRole>().eq("user_id",userId));
        }
        String[] array = role.toArray(new String[role.size()]);
        for (String id:array){
            SysUserRole sysRole= new SysUserRole();
            sysRole.setRoleId(Integer.valueOf(id)).setUserId(userId);
            sysUserRoleMapper.insert(sysRole);
        }
        responseBean.addSuccess("成功");
        return responseBean;
    }

    @Override
    public ResponseBean roleAll() {
       ResponseBean responseBean=  new ResponseBean<>();
        List<SysRole> sysRole= sysRoleMapper.selectroleAll();
        responseBean.addSuccess(sysRole);
        return responseBean;
    }

    @Override
    public ResponseBean deleteUserRole(Integer userId) {
        ResponseBean responseBean= new ResponseBean<>();
        QueryWrapper<SysUserRole> sysUserRoleQueryWrapper = new QueryWrapper<>();
        sysUserRoleQueryWrapper.eq("user_id",userId);
        int sysUserRole=  sysUserRoleMapper.delete(sysUserRoleQueryWrapper);
        responseBean.addSuccess(sysUserRole);
        return responseBean;
    }

    @Override
    public ResponseBean sysmenu(IPage<SysMenu> pg) {
        ResponseBean responseBean = new ResponseBean();
        IPage<SysMenu> list = sysMenuMapper.selectSysMenu(pg);
        responseBean.addSuccess(list);
        return responseBean;
    }

    @Override
    public ResponseBean sysmenuAdd(String menuId,String menuName,String code) {
        ResponseBean responseBean= new ResponseBean<>();
        QueryWrapper<SysMenu> sysMenuQuery= new QueryWrapper<>();
        sysMenuQuery.eq("menu_id",menuId);
        // sysMenu 不为空时是父节点信息
        SysMenu sysMenu= sysMenuMapper.selectOne(sysMenuQuery);
        if ("".equals(code)&& null!=code){
            sysMenu.setMenuName(menuName);
            responseBean.addSuccess("成功");
            sysMenuMapper.updateSysMenu(menuId,menuName);
            return responseBean;
        }
        SysMenu menu = new SysMenu();
                menu.setMenuId(menuId+"")
                        .setMenuName(menuName)
                        .setMenuPid(sysMenu.getMenuId())
                        .setMenuParents(menuId+","+sysMenu.getMenuId())
                        .setStatus(1).setCreateDate(LocalDateTime.now());
            sysMenuMapper.insert(menu);
            responseBean.addSuccess("成功");
            return responseBean;
    }

    @Override
    public ResponseBean fireUser(Integer userId) {
        ResponseBean responseBean= new ResponseBean<>();
        sysUserMapper.fireUser(userId);
        responseBean.addSuccess("成功");
        return responseBean;
    }
}
